package com.xydl.cac.task;

import com.xydl.cac.entity.*;
import com.xydl.cac.model.i2sync.*;
import com.xydl.cac.repository.I2syncRecordRepository;
import com.xydl.cac.repository.NSensorRepository;
import com.xydl.cac.service.DataService;
import com.xydl.cac.service.I2syncService;
import com.xydl.cac.util.DateUtil;
import com.xydl.cac.util.JSONUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

@Service
@Slf4j
public class I2syncTask {
    @Resource
    I2syncService i2syncService;
    @Resource
    I2syncRecordRepository recordRepository;
    @Resource
    NSensorRepository sensorRepository;
    @Resource
    DataService dataService;

    @Scheduled(initialDelay = 60 * 1000, fixedDelay = 30 * 60 * 1000)
    public void syncAll() {
        List<I2syncConfig> configList = i2syncService.listConfig();
        if (!CollectionUtils.isEmpty(configList)) {
            log.info("I2syncTask.syncAll 开始.");
            try {
                for (I2syncConfig config : configList) {
                    this.syncConfig(config);
                }
                log.info("I2syncTask.syncAll 结束.");
            } catch (Exception e) {
                log.error("I2syncTask.syncAll error.", e);
            }
        }
    }

    // 同步一个类型
    private void syncConfig(I2syncConfig config) throws Exception {
        List<I2syncField> fieldList = i2syncService.listFieldConfig(config.getTableName());
        if (CollectionUtils.isEmpty(fieldList)) {
            return;
        }
        List<ModevTypePoint> points = new ArrayList<>();
        for (I2syncField field : fieldList) {
            ModevTypePoint point = new ModevTypePoint();
            point.setField(field.getFieldName());
        }
        List<NSensor> list = sensorRepository.findByTypeId(config.getModevtypeId());
        for (NSensor sensor : list) {
            this.syncSensor(sensor, fieldList, points);
        }
    }

    // 同步一个装置
    private void syncSensor(NSensor sensor, List<I2syncField> fieldList, List<ModevTypePoint> points) throws Exception {
        I2syncRecord record;
        List<I2syncRecord> recordList = recordRepository.findByEqmid(sensor.getDevId());
        if (CollectionUtils.isEmpty(recordList)) {
            record = new I2syncRecord();
            record.setEqmid(sensor.getDevId());
            record.setModevtypeId(sensor.getTypeId());
        } else {
            record = recordList.get(0);
        }

        // 查询最新数据
        List<Map<String, Object>> list = dataService.getLatestData(sensor, points, record.getLastDTime());
        if (CollectionUtils.isEmpty(list)) {
            return;
        }

        for (Map<String, Object> map : list) {
            String date = (String) map.get("acquisitionTime");
            // 格式转换
            Request request = this.transform(sensor, fieldList, map, date);
            String xml = JSONUtil.object2Xml(request);

            // 同步导出

            record.setLastDTime(DateUtil.parse(date));
            recordRepository.save(record);
        }
    }

    private Request transform(NSensor sensor, List<I2syncField> fieldList, Map<String, Object> map, String date) {
        List<Attr> attrs = new ArrayList<>();
        for (I2syncField field : fieldList) {
            Object value = map.get(field.getFieldName());
            Attr attr = new Attr();
            attr.setName(field.getDestFieldName());
            attr.setValue(String.valueOf(value));
            attr.setAlarm("FALSE");
            attrs.add(attr);
        }
        Datanode node = new Datanode();
        node.setSensorid(sensor.getEquipmentId());
        node.setType("027002");
        node.setEquipmentid(sensor.getEquipmentId());
        node.setTimestamp(date);
        node.setAttr(attrs);
        Monitordata monitordata = new Monitordata();
        monitordata.setCacid(sensor.getEquipmentId());
        monitordata.setDatanodenum(1);
        monitordata.setDatanode(node);
        Request request = new Request();
        request.setMonitordata(monitordata);
        return request;
    }
}
